@@ -5,7 +5,8 @@ from urllib.parse import quote as _uriquote
 import requests
 
 from . import __version__, errors, utils
-from .checks import _county_types, _leaderboard_types, _vpn_types, _notNone_check
+from .converters import _county_types, _leaderboard_types, _vpn_types, _not_none
+from . import checks
 from .cog import request_cog
 
 GET='get'
@@ -14,11 +15,13 @@ POST='post'
 
 class HTTPClient:
     __CSRF_token_regex = re.compile("const csrfToken[ ]{0,1}=[ ]{0,1}[\"|'](.{36})[\"|']")
-    __Username_regex = re.compile("const username[ ]{0,1}=[ ]{0,1}[\"|'](.{1,16})[\"|']")
+    __Username_regex   = re.compile("const username[ ]{0,1}=[ ]{0,1}[\"|'](.{1,16})[\"|']")
+    
     def __init__(self, session=None):
+        self._state = None
         self.authenticated = False
         self.__session = requests.Session()
-        self.satic_session = requests.Session()
+        self.static_session = requests.Session()
         self.connect_sid = None
         self._CSRF_token = None
         self.username = None
@@ -70,10 +73,6 @@ class HTTPClient:
         method = route.method
         settings = kwargs.pop('settings', {})
         
-        if settings.get("static", False):
-            session = self.satic_session
-        
-        
         headers = {
             'User-Agent': self.user_agent
         }
@@ -81,6 +80,12 @@ class HTTPClient:
         if 'json' in kwargs:
             headers['Content-Type'] = 'application/json'
             kwargs['data'] = utils.to_json(kwargs.pop('json'))
+        
+        if "static" in settings:
+            session = self.static_session
+        if "CSRF" in settings:
+            headers['CSRF-Token'] = self._CSRF_token
+            kwargs["data"]["_CSRF"] = self._CSRF_token
 
         # TODO: retries, Pagenator
         try:
@@ -186,7 +191,7 @@ class RouteList:
     def get_user_games(      **parameters):  return Route(path="/games/koth/user/games",                 **parameters)
     def get_game_tickets_won(**parameters):  return Route(path="/games/tickets/won?username={username}", **parameters)
     def post_join_koth(      **parameters):  return Route(method=POST, path="/games/koth/new",           **parameters) 
-    def post_new_koth(       **parameters):  return Route(method=POST, path="/games/koth/join-public",   **parameters) # ? might change for premium
+    def post_new_koth(       **parameters):  return Route(method=POST, path="/games/koth/join-public",   **parameters) # ? might be different for premium users
     
     # * VPN
     
@@ -245,7 +250,7 @@ class RouteList:
     def get_room_scoreboard(     **parameters): return Route(path="/api/room/scoreboard?code={room_code}", **parameters)
     def get_room_votes(          **parameters): return Route(path="/api/room/votes?code={room_code}",      **parameters)
     def get_room_details(        **parameters): return Route(path="/api/room/details?codes={room_code}",   **parameters) # ? list posibility
-    def get_room_tasks(          **parameters): return Route(path="/api/tasks/{room_code}",           **parameters)
+    def get_room_tasks(          **parameters): return Route(path="/api/tasks/{room_code}",                **parameters)
     def post_room_answer(        **parameters): return Route(method=POST, path="/api/{room_code}/answer",  **parameters)
     def post_deploy_machine(     **parameters): return Route(method=POST, path="/material/deploy",         **parameters)
     def post_reset_room_progress(**parameters): return Route(method=POST, path="/api/reset-progress",      **parameters)
@@ -255,168 +260,189 @@ class RouteList:
 class HTTP(request_cog, HTTPClient):
     # * normal site calls
     
-    def get_server_time(self):
-        return self.request(RouteList.get_server_time())
-    def get_site_stats(self):
-        return self.request(RouteList.get_site_stats())
-    def get_practise_rooms(self): 
-        return self.request(RouteList.get_practise_rooms())
-    def get_serie(self, show, serie_code):
-        return self.request(RouteList.get_series(show=show, options={"name": serie_code}))
-    def get_series(self, show):
-        return self.request(RouteList.get_series(show=show))
-    def get_glossary_terms(self): 
-        return self.request(RouteList.get_glossary_terms())
+    def get_server_time(self, **attrs):
+        return self.request(RouteList.get_server_time(), **attrs)
+    def get_site_stats(self, **attrs):
+        return self.request(RouteList.get_site_stats(), **attrs)
+    def get_practise_rooms(self, **attrs): 
+        return self.request(RouteList.get_practise_rooms(), **attrs)
+    def get_serie(self, show, serie_code, **attrs):
+        return self.request(RouteList.get_series(show=show, options={"name": serie_code}), **attrs)
+    def get_series(self, show, **attrs):
+        return self.request(RouteList.get_series(show=show), **attrs)
+    def get_glossary_terms(self, **attrs): 
+        return self.request(RouteList.get_glossary_terms(), **attrs)
     
     # * Leaderboards
-    
-    def get_leaderboards(self, country: _county_types=None, type:_leaderboard_types=None):
-        return self.request(RouteList.get_leaderboards(country=country.to_lower_case(), type=type))
-    def get_koth_leaderboards(self, country: _county_types=None, type:_leaderboard_types=None):
-        return self.request(RouteList.get_koth_leaderboards(country=country.to_lower_case(), type=type))
+    def get_leaderboards(self, country: _county_types, type:_leaderboard_types, **attrs):
+        return self.request(RouteList.get_leaderboards(country=country.to_lower_case(), type=type), **attrs)
+    def get_koth_leaderboards(self, country: _county_types, type:_leaderboard_types, **attrs):
+        return self.request(RouteList.get_koth_leaderboards(country=country.to_lower_case(), type=type), **attrs)
 
     # * networks
     
-    def get_network(self, network_code): 
-        return self.request(RouteList.get_network(network_code=network_code))
-    def get_networks(self): 
-        return self.request(RouteList.get_networks())
-    def get_network_cost(self, network_code): 
-        return self.request(RouteList.get_networks(network_code=network_code))
+    def get_network(self, network_code, **attrs): 
+        return self.request(RouteList.get_network(network_code=network_code), **attrs)
+    def get_networks(self, **attrs): 
+        return self.request(RouteList.get_networks(),**attrs)
+    def get_network_cost(self, network_code, **attrs): 
+        return self.request(RouteList.get_networks(network_code=network_code), **attrs)
     
     # * account
-    
-    def get_subscription_cost(self): 
-        return self.request(RouteList.get_subscription_cost())
+    @checks.is_authenticated()
+    def get_subscription_cost(self, **attrs): 
+        return self.request(RouteList.get_subscription_cost(), **attrs)
     
     # * paths
     
-    def get_path(self, path_code):
-        return self.request(RouteList.get_path(path_code=path_code))
-    def get_public_paths(self):
-        return self.request(RouteList.get_public_paths())
-    def get_path_summary(self):
-        return self.request(RouteList.get_path_summary())
+    def get_path(self, path_code, **attrs):
+        return self.request(RouteList.get_path(path_code=path_code), **attrs)
+    def get_public_paths(self, **attrs):
+        return self.request(RouteList.get_public_paths(), **attrs)
+    def get_path_summary(self, **attrs):
+        return self.request(RouteList.get_path_summary(), **attrs)
     
     # * modules
     
-    def get_modules_summary(self):
-        return self.request(RouteList.get_modules_summary())
-    def get_module(self, module_code):
-        return self.request(RouteList.get_module(module_code))
+    def get_modules_summary(self, **attrs):
+        return self.request(RouteList.get_modules_summary(), **attrs)
+    def get_module(self, module_code, **attrs):
+        return self.request(RouteList.get_module(module_code), **attrs)
     
     # * games
     
-    def get_machine_pool(self):
-        return self.request(RouteList.get_machine_pool())
-    def get_game_detail(self, game_code):
-        return self.request(RouteList.get_game_detail(game_code=game_code))
-    def get_recent_games(self):
-        return self.request(RouteList.get_recent_games())
-    def get_user_games(self):
-        return self.request(RouteList.get_user_games())
-    def get_game_tickets_won(self, username):
-        return self.request(RouteList.get_game_tickets_won(username=username))
-    def post_join_koth(self):
-        return self.request(RouteList.post_join_koth())
-    def post_new_koth(self):
-        return self.request(RouteList.post_new_koth())
+    def get_machine_pool(self, **attrs):
+        return self.request(RouteList.get_machine_pool(), **attrs)
+    def get_game_detail(self, game_code, **attrs):
+        return self.request(RouteList.get_game_detail(game_code=game_code), **attrs)
+    def get_recent_games(self, **attrs):
+        return self.request(RouteList.get_recent_games(), **attrs)
+    def get_user_games(self, **attrs):
+        return self.request(RouteList.get_user_games(), **attrs)
+    def get_game_tickets_won(self, username, **attrs):
+        return self.request(RouteList.get_game_tickets_won(username=username), **attrs)
+    @checks.set_header_CSRF()
+    def post_join_koth(self, **attrs):
+        return self.request(RouteList.post_join_koth(), **attrs)
+    @checks.set_header_CSRF()
+    def post_new_koth(self, **attrs):
+        return self.request(RouteList.post_new_koth(), **attrs)
     
     # * VPN
-    
-    def get_available_vpns(self, type : _vpn_types):
-        return self.request(RouteList.get_available_vpns(options={"type": type}))
-    def get_vpn_info(self):
-        return self.request(RouteList.get_vpn_info())
+    @checks.is_authenticated()
+    def get_available_vpns(self, type : _vpn_types, **attrs):
+        return self.request(RouteList.get_available_vpns(options={"type": type}), **attrs)
+    @checks.is_authenticated()
+    def get_vpn_info(self, **attrs):
+        return self.request(RouteList.get_vpn_info(), **attrs)
     
     # * VM
 
-    def get_machine_running(self):
-        return self.request(RouteList.get_machine_running())
-    def post_renew_machine(self, room_code):
-        return self.request(RouteList.post_renew_machine(), json={"code": room_code, "_csrf": ""})
-    def post_terminate_machine(self, room_code):
-        return self.request(RouteList.post_terminate_machine(), json={"code": room_code, "_csrf": ""})
+    def get_machine_running(self, **attrs):
+        return self.request(RouteList.get_machine_running(), **attrs)
+    @checks.set_header_CSRF()
+    def post_renew_machine(self, room_code, **attrs):
+        return self.request(RouteList.post_renew_machine(), json={"code": room_code}, **attrs)
+    @checks.set_header_CSRF()
+    def post_terminate_machine(self, room_code, **attrs):
+        return self.request(RouteList.post_terminate_machine(), json={"code": room_code}, **attrs)
     
     # * user -badge
-    
-    def get_own_badges(self):
-        return self.request(RouteList.get_own_badges())
-    def get_user_badges(self, username):
-        return self.request(RouteList.get_user_badges(username=username))
-    def get_all_badges(self):
-        return self.request(RouteList.get_all_badges())
+    @checks.is_authenticated()
+    def get_own_badges(self, **attrs):
+        return self.request(RouteList.get_own_badges(), **attrs)
+    def get_user_badges(self, username, **attrs):
+        return self.request(RouteList.get_user_badges(username=username), **attrs)
+    def get_all_badges(self, **attrs):
+        return self.request(RouteList.get_all_badges(), **attrs)
     
     # * user -team
-    
-    def get_team_info(self): 
-        return self.request(RouteList.get_team_info())
+    @checks.is_authenticated()
+    def get_team_info(self, **attrs): 
+        return self.request(RouteList.get_team_info(), **attrs)
     
     # * user -notifications
-    
-    def get_unseen_notifications(self):
-        return self.request(RouteList.get_unseen_notifications())
-    def get_all_notifications(self):
-        return self.request(RouteList.get_all_notifications())
+    @checks.is_authenticated()
+    def get_unseen_notifications(self, **attrs):
+        return self.request(RouteList.get_unseen_notifications(), **attrs)
+    @checks.is_authenticated()
+    def get_all_notifications(self, **attrs):
+        return self.request(RouteList.get_all_notifications(), **attrs)
 
     # * user -messages
 
-    def get_unseen_messages(self):
-        return self.request(RouteList.get_unseen_messages())
-    def get_all_group_messages(self):
-        return self.request(RouteList.get_all_group_messages())
-    def get_group_messages(self, group_id):
-        return self.request(RouteList.get_group_messages(group_id))
+    @checks.is_authenticated()
+    def get_unseen_messages(self, **attrs):
+        return self.request(RouteList.get_unseen_messages(), **attrs)
+    @checks.is_authenticated()
+    def get_all_group_messages(self, **attrs):
+        return self.request(RouteList.get_all_group_messages(), **attrs)
+    @checks.is_authenticated()
+    def get_group_messages(self, group_id, **attrs):
+        return self.request(RouteList.get_group_messages(group_id), **attrs)
     
     # * user -room
 
-    def get_user_completed_rooms_count(self, username):
-        return self.request(RouteList.get_user_completed_rooms_count(username=username))
-    def get_user_completed_rooms(self, username, limit:int=10, page:int=1):
-        return self.request(RouteList.get_user_completed_rooms(username=username, options={"limit": limit, "page": page}))
-    def get_user_created_rooms(self, username, limit:int=10, page:int=1):
-        return self.request(RouteList.get_user_created_rooms(username=username, options={"limit": limit, "page": page}))
+    def get_user_completed_rooms_count(self, username, **attrs):
+        return self.request(RouteList.get_user_completed_rooms_count(username=username), **attrs)
+    def get_user_completed_rooms(self, username, limit:int=10, page:int=1, **attrs):
+        return self.request(RouteList.get_user_completed_rooms(username=username, options={"limit": limit, "page": page}), **attrs)
+    def get_user_created_rooms(self, username, limit:int=10, page:int=1, **attrs):
+        return self.request(RouteList.get_user_created_rooms(username=username, options={"limit": limit, "page": page}), **attrs)
 
     # * user
 
-    def get_user_rank(self, username : _notNone_check):
-        return self.request(RouteList.get_user_rank(username=username))
-    def get_user_activty(self, username : _notNone_check):
-        return self.request(RouteList.get_user_activty(username=username))
-    def get_all_friends(self):
-        return self.request(RouteList.get_all_friends())
-    def get_discord_user(self, username : _notNone_check):
-        return self.request(RouteList.get_discord_user(username=username))
-    def get_user_exist(self, username : _notNone_check):
-        return self.request(RouteList.get_user_exist(username=username))
-    def search_user(self, username : _notNone_check):
-        return self.request(RouteList.search_user(username=username))
+    def get_user_rank(self, username : _not_none, **attrs):
+        return self.request(RouteList.get_user_rank(username=username), **attrs)
+    def get_user_activty(self, username : _not_none, **attrs):
+        return self.request(RouteList.get_user_activty(username=username), **attrs)
+    @checks.is_authenticated()
+    def get_all_friends(self, **attrs):
+        return self.request(RouteList.get_all_friends(), **attrs)
+    def get_discord_user(self, username : _not_none, **attrs):
+        return self.request(RouteList.get_discord_user(username=username), **attrs)
+    def get_user_exist(self, username : _not_none, **attrs):
+        return self.request(RouteList.get_user_exist(username=username), **attrs)
+    def search_user(self, username : _not_none, **attrs):
+        return self.request(RouteList.search_user(username=username), **attrs)
 
     # * room
 
-    def get_new_rooms(self):
-        return self.request(RouteList.get_new_rooms())
-    def get_recommended_rooms(self):
-        return self.request(RouteList.get_recommended_rooms())
-    def get_questions_answered(self):
-        return self.request(RouteList.get_questions_answered())
-    def get_joined_rooms(self):
-        return self.request(RouteList.get_joined_rooms())
-    def get_room_percentages(self, room_codes):
-        return self.request(RouteList.get_room_percetages(), json={"rooms": room_codes})
-    def get_room_scoreboard(self, room_code):
-        return self.request(RouteList.get_room_scoreboard(room_code=room_code))
-    def get_room_votes(self, room_code):
-        return self.request(RouteList.get_room_votes(room_code=room_code))
-    def get_room_details(self, room_code, loadWriteUps: bool=True, loadCreators: bool=True, loadUser: bool=True):
-        return self.request(RouteList.get_room_details(room_code=room_code, options={"loadWriteUps": loadWriteUps, "loadCreators": loadCreators, "loadUser": loadUser})).get(room_code)
-    def get_room_tasks(self, room_code, **request_options):
-        return self.request(RouteList.get_room_tasks(room_code=room_code), **request_options)
-    def post_room_answer(self, room_code, taskNo: int, questionNo: int, answer: str):
-        return self.request(RouteList.post_room_answer(room_code=room_code), json={"_csrf": "", "taskNo": taskNo, "questionNo": questionNo, "answer": answer})
-    def post_deploy_machine(self, room_code, uploadId): # ? csrf token
-        return self.request(RouteList.post_deploy_machine(), json={"roomCode": room_code, "id": uploadId, "_csrf": ""})
-    def post_reset_room_progress(self, room_code): # ? csrf token
-        return self.request(RouteList.post_reset_room_progress(), json={"code": room_code, "_csrf": ""})
-    def post_leave_room(self, room_code): # ? csrf token
-        return self.request(RouteList.post_leave_room(), json={"code": room_code, "_csrf": ""})
+    def get_new_rooms(self, **attrs):
+        return self.request(RouteList.get_new_rooms(), **attrs)
+    @checks.is_authenticated()
+    def get_recommended_rooms(self, **attrs):
+        return self.request(RouteList.get_recommended_rooms(), **attrs)
+    def get_questions_answered(self, **attrs):
+        return self.request(RouteList.get_questions_answered(), **attrs)
+    @checks.is_authenticated()
+    def get_joined_rooms(self, **attrs):
+        return self.request(RouteList.get_joined_rooms(), **attrs)
+    @checks.is_authenticated()
+    def get_room_percentages(self, room_codes, **attrs):
+        return self.request(RouteList.get_room_percetages(), json={"rooms": room_codes}, **attrs)
+    @checks.is_authenticated()
+    def get_room_scoreboard(self, room_code, **attrs):
+        return self.request(RouteList.get_room_scoreboard(room_code=room_code), **attrs)
+    def get_room_votes(self, room_code, **attrs):
+        return self.request(RouteList.get_room_votes(room_code=room_code), **attrs)
+    def get_room_details(self, room_code, loadWriteUps: bool=True, loadCreators: bool=True, loadUser: bool=True, **attrs):
+        return self.request(RouteList.get_room_details(room_code=room_code, options={"loadWriteUps": loadWriteUps, "loadCreators": loadCreators, "loadUser": loadUser}), **attrs).get(room_code, {})
+    def get_room_tasks(self, room_code, **attrs):
+        return self.request(RouteList.get_room_tasks(room_code=room_code), **attrs)
+    @checks.set_header_CSRF()
+    @checks.is_authenticated()
+    def post_room_answer(self, room_code, taskNo: int, questionNo: int, answer: str, **attrs):
+        return self.request(RouteList.post_room_answer(room_code=room_code), json={"taskNo": taskNo, "questionNo": questionNo, "answer": answer}, **attrs)
+    @checks.set_header_CSRF()
+    @checks.is_authenticated()
+    def post_deploy_machine(self, room_code, uploadId, **attrs):
+        return self.request(RouteList.post_deploy_machine(), json={"roomCode": room_code, "id": uploadId}, **attrs)
+    @checks.set_header_CSRF()
+    @checks.is_authenticated()
+    def post_reset_room_progress(self, room_code, **attrs):
+        return self.request(RouteList.post_reset_room_progress(), json={"code": room_code}, **attrs)
+    @checks.set_header_CSRF()
+    @checks.is_authenticated()
+    def post_leave_room(self, room_code, **attrs):
+        return self.request(RouteList.post_leave_room(), json={"code": room_code}, **attrs)
